#pragma once
#include <QString>

#include <QPointF>
#include <QRectF>
class ActionTypes {

public:

    static QString value1;

    static int value2;

    static qreal value3;

    static bool value4;

    static bool value4b;

    static QPointF value5;

    static QRectF value6;

};
