#include "actiontypes.h"

QString ActionTypes::value1 = "value1";

int ActionTypes::value2 = 2;

qreal ActionTypes::value3 = 3.3;

bool ActionTypes::value4 = true;

bool ActionTypes::value4b = false;

QPointF ActionTypes::value5 = QPointF(5,6);

QRectF ActionTypes::value6 = QRect(6,7,8,9);
